@@ -3,15 +3,12 @@
 
 #include <memory> // *_ptr
 
-#include <RcppArmadillo.h>
+#define R_NO_REMAP
 #include <RcppParallel.h>
+#include <Rinternals.h>
 
 namespace dtwclust {
 
-// =================================================================================================
-/* Distmat (base + factory) */
-// =================================================================================================
-
 // -------------------------------------------------------------------------------------------------
 /* abstract distmat */
 // -------------------------------------------------------------------------------------------------
@@ -20,6 +17,8 @@ class Distmat
 public:
     virtual ~Distmat() {};
     virtual double& operator() (const int i, const int j) = 0;
+    virtual int nrow() const = 0;
+    virtual int ncol() const = 0;
 };
 
 // -------------------------------------------------------------------------------------------------
@@ -31,6 +30,21 @@ public:
     std::shared_ptr<Distmat> create(const SEXP& MAT_TYPE, const SEXP& D);
 };
 
+// -------------------------------------------------------------------------------------------------
+/* R matrix distmat (thread-safe) */
+// -------------------------------------------------------------------------------------------------
+class RDistmat : public Distmat
+{
+public:
+    RDistmat(const SEXP& D);
+    double& operator() (const int i, const int j) override;
+    int nrow() const override;
+    int ncol() const override;
+
+private:
+    RcppParallel::RMatrix<double> distmat_;
+};
+
 } // namespace dtwclust
 
 #endif // DTWCLUST_DISTMAT_HPP_
